#calculate deep learning feature maps

import numpy as np
from DenseNet_19 import DenseNet, data_preprocess, get_model_files
from keras import backend as K
import tensorflow as tf
from scipy.stats import spearmanr
import glob

path = ''

for CT_phase_num in range(3):
    CT_phase = ['A','V','p'][CT_phase_num]
    
    model_files = get_model_files()
    model_file = model_files[CT_phase_num]
    print(model_file)
    
    weights1 = np.load(path+'/weights-'+CT_phase+'2-01L1L2.npy');weights1_w = weights1[0]
    GAPs = np.load(path+'/GAP-'+CT_phase+'.npz')
    label_train = GAPs['label_train'];preds_train = GAPs['preds_train']#training set in primary cohort
    label_valid = GAPs['label_valid'];preds_valid = GAPs['preds_valid']#validation set in primary cohort
    candidata_nodes = np.where(np.abs(weights1_w) > 0.01)[0]
    corrs = np.zeros(len(candidata_nodes))
    for i in range(len(candidata_nodes)):
        corrs[i] = spearmanr(preds_valid[:,candidata_nodes[i]], label_valid[:,1])[1]
    candidata_nodes = candidata_nodes[np.where(corrs<0.05)[0]]
    print('candidata_nodes (significant) number: ',len(candidata_nodes))
    
    node_sort = np.argsort(np.abs(weights1_w)[candidata_nodes,0])
    node_sort = node_sort[::-1]
    selected_nodes = np.array([candidata_nodes[node_sort[0]]])
    for i in range(len(candidata_nodes)-1):
        t = 0
        for j in range(len(selected_nodes)):
            if spearmanr(preds_train[:,candidata_nodes[node_sort[i+1]]], preds_train[:,selected_nodes[j]])[1]>0.6:
                t = 1
        if t == 0:
            selected_nodes = np.append(selected_nodes, candidata_nodes[node_sort[i+1]])
    print('selected_nodes (non-redundant) number: ',len(selected_nodes))
    key_nodes = selected_nodes[:8]
    
    print(key_nodes)
    print(weights1_w[key_nodes])
    
    model = DenseNet([6, 12, 48, 32], include_top=False,
                     input_tensor=None, input_shape=None, pooling=None)
    model.load_weights(path+'/'+model_file, by_name=True)
    
    path_slice = glob.glob(path+'/slice_'+CT_phase+'*')
    print(path_slice)
    for i in range(np.shape(path_slice)[0]):#
        cohort = path_slice[i][-6:-4]
        print(cohort)
        data1 = np.load(path+'/slice_'+CT_phase+'_'+cohort+'.npy')
        print('DATA LOAD READY.')
        print(data1.shape)
        
        dl_feature = None
        for j in range(data1.shape[0]):
            if j % 20 == 0:
                print(j)
            data2 = data_preprocess(data1[j,:])
            dl_feature2 = model.predict(data2)
            if dl_feature is None:
                dl_feature = dl_feature2[:,:,:,key_nodes]
            else:
                dl_feature = np.vstack((dl_feature, dl_feature2[:,:,:,key_nodes]))
        print(dl_feature.shape)
        
        np.save(path+'/dl_feature/'+CT_phase+'_'+cohort+'.npy',dl_feature)
        
    K.clear_session()
    tf.reset_default_graph()